/**\
|**| =====================================================================
|**|
|**|	Tracking Layout.c
|**|
|**|	This file contains the calls that this sample needs to make
|**|	the QuickDraw GX shell work correctly.
|**|
|**|	QuickDraw GX Libraries Used:
|**|	"ColorLibrary.c", "FontLibrary.c", "GraphicsDebugLibrary.c",
|**|	"LayoutLibrary.c", "ShapeLibrary.c", and "TransformLibrary.c".
|**|
|**|	6/96 bob	Updated #includes to support changed GX Library names.
|**|				Updated the copyright date.
|**|
|**|	1990 - 1996  Apple Computer, Inc.
|**|	All rights reserved.
|**|
|**| =====================================================================
\**/


#include "QDGX shell.h"
#include <GXLayout.h>
#include "LayoutLibrary.h"


/**\
|**| ---------------------------------------------------------------------
|**| PROTOTYPES
|**| ---------------------------------------------------------------------
\**/

// funtions required by shell

void	DoSetup			(void);
void	DoDraw			(WindowPtr wind, Boolean updating);
OSErr	DoCreateNew		(void);
void	DoDispose		(WindowPtr wind);
void	DoIdle			(WindowPtr wind);
void	DoTeardown		(void);
void	DoClick			(WindowPtr wind, Point p);

// private functions

OSErr	DoWindowInit		(WindowPtr wind);
void	CreateSampleImage	(WindowPtr wind);


/**\
|**| ---------------------------------------------------------------------
|**| ENUMS
|**| ---------------------------------------------------------------------
\**/
enum { rWindResource = 128 };


/**\
|**| ---------------------------------------------------------------------
|**| GLOBALS
|**| ---------------------------------------------------------------------
\**/
// If gDebugging = TRUE, graphics library errors and notices will be posted.  This
// functionality will only work with the "debugging" version of QuickDraw GX.
// If the debugging version is not installed, nothing bad will happen, but these
// functions will not work. 

Boolean		gDebugging = true;

// Set  "gGiveMeValidation" to TRUE if you want receive run-time validation.

Boolean		gGiveMeValidation = true;


// gGraphicsHeapSize sets the size of the graphics heap created by calling the
// GXNewGraphicsClient routine in main () within QuickDraw GX shell.c.  You can determine
// the amount of graphics heap required by using GraphicsBug.  I'm giving it 600K,
// since we need abunch if we have several windows open.

long		gGraphicsHeapSize = 600;

// gOurPrintingOverrideUPP is a universal proc pointer for our printing event
// override.  This is so that our override can be native PowerPC code if necessary.

GXPrintingEventUPP	gOurPrintingOverrideUPP;



/**\
|**| ---------------------------------------------------------------------
|**| DoSetup()
|**| Here's where we initialize any global variables our application needs.
|**| We have only one at this time -- the universal proc pointer for
|**| our printing override.
|**| ---------------------------------------------------------------------
\**/
void DoSetup (void)
{	// Initialize our printing event override UPP
	gOurPrintingOverrideUPP = NewGXPrintingEventProc(MyPrintingEventOverride);
}


/**\
|**| ---------------------------------------------------------------------
|**| DoDraw()
|**| Draw the contents of the window.  The first parameter is the window
|**| to draw, and the second parameter is true if we're updating an existing
|**| image.  If that's the case, we don't want to change anything, but
|**| just draw what's already there.
|**| ---------------------------------------------------------------------
\**/
void DoDraw (WindowPtr wind, Boolean updating)
{
	 #pragma unused (updating)
	 GXDrawShape (GetDocShape(wind));
}


/**\
|**| ---------------------------------------------------------------------
|**| DoCreateNew()
|**| This routine is called when a window needs to be created.
|**| ---------------------------------------------------------------------
\**/
OSErr DoCreateNew (void)
{
	OSErr		err = noErr;
	WindowPtr	wind;
	
// Get and create our window from the resource fork

	wind = GetNewWindow(rWindResource, nil, (WindowPtr)-1L);

// Attach a default gxViewPort to it, create and iInitialize our
// private data for it, and add a sample image to its page shape.

	if ( wind == NULL )
		return (MemError());

	GXIgnoreGraphicsNotice(transform_already_set);
	SetDefaultViewPort(GXNewWindowViewPort(wind));			
	GXPopGraphicsNotice();
	
	err = DoWindowInit(wind);
	if ( err != noErr )
		return err;
	
	CreateSampleImage(wind);
	return err;
}


/**\
|**| ---------------------------------------------------------------------
|**| DoDispose()
|**| This routine is called when a window needs to be disposed of.
|**| ---------------------------------------------------------------------
\**/
void DoDispose (WindowPtr wind)
{
	TH_Doc	doc;
	
// You should always dispose of your GX graphics objects before tossing your window.
// Why?  It's generally good form and this approach guarantees that everything is
// disposed.  If you had not disposed of everything, the call to DisposeWindow should
// dispose of the objects. If you are running the debugging version of QuickDraw GX
// with notices set, you will receive a notice that you had not disposed of everything.
// You can turn notices on in this file by setting gDebugging = TRUE (above).
	
	if ( wind != NULL )
	{
		doc = (TH_Doc)GetWRefCon(wind);		// Remember, this is where we stored our private data.
		GXDisposeShape(GetDocShape(wind)); 	// Dispose of this doc's shape.
		GXDisposeJob(GetDocJob(wind));		// Dispose of this doc's print job.
		DisposHandle((Handle) doc);			// Dispose of our private data.
		DisposeWindow(wind);				// Dispose of the window.
	}
}


/**\
|**| ---------------------------------------------------------------------
|**| DoIdle()
|**| This routine is called to do things while idling through the event loop.
|**| ---------------------------------------------------------------------
\**/
void DoIdle (WindowPtr wind)
{
}


/**\
|**| ---------------------------------------------------------------------
|**| DoTeardown()
|**| This routine is called just before we quit to remove anything 
|**| persistent that might have been setup by DoSetup().
|**| ---------------------------------------------------------------------
\**/
void DoTeardown (void)
{
	DisposeRoutineDescriptor(gOurPrintingOverrideUPP);
}


/**\
|**| ---------------------------------------------------------------------
|**| DoClick()
|**| ---------------------------------------------------------------------
\**/
void DoClick(WindowPtr window, Point p)
{
}





/**\
|**| ---------------------------------------------------------------------
|**| DoWindowInit()
|**| In this function we create and initialize the the private document
|**| structure for a new window.  This structure contains the print job and
|**| the shape which is drawn in the window.  We store this data in a handle
|**| and hang it off the window's refCon field for easy retrieval.  By doing
|**| this, rather than using globals, we can create many windows containing
|**| unique print jobs and shapes.
|**| ---------------------------------------------------------------------
\**/
OSErr DoWindowInit (WindowPtr wind)
{
	OSErr	err = noErr;
	gxJob	docJob;
	gxShape	docPage;
	TH_Doc	windDoc;


// Create the page shape. We set the unique items attribute to make sure that each item
// added to the picture has a unique reference. If this attribute was not set, we would
// not see all copies of anything we add to the shape multiple times -- we'd just see
// the last version added.		

	docPage = GXNewShape(gxPictureType);
	GXSetShapeAttributes(docPage, (GXGetShapeAttributes(docPage) | gxUniqueItemsShape));
	
	
// Create a print job for this document.  This will be the same as the system default until
// the user goes through the dialogs for Page Setup or Print

	err = GXNewJob(&docJob);
	
	
// If there are no errors, create a handle the size of our document structure and store
// the print job and page shape in it.  Store the handle in the window's refCon field so
// that we can get at it.  (Note that the utility routines "GetDocJob" and "GetDocShape"
// can be used to do this easily.

	if ( err == noErr )
	{
		windDoc = (TH_Doc) NewHandleClear(sizeof(T_Doc));

		if ( windDoc == NULL )
			err = MemError();
		else
		{
			(*windDoc)->docJob = docJob;
			(*windDoc)->docPage = docPage;
			SetWRefCon(wind, (long) windDoc);
		}

// Now install our application override for PrintingEvent so that we can
// support the new movable-modal printing dialog boxes.

		GXInstallApplicationOverride(docJob, gxPrintingEventMsg, gOurPrintingOverrideUPP);

	}

	return err;
}


/**\
|**| ---------------------------------------------------------------------
|**| CreateSampleImage()
|**| This function creates primitive shapes and adds them to the window's page shape.
|**| ---------------------------------------------------------------------
\**/
void CreateSampleImage (WindowPtr wind)
{
	Rect	ourWindowRect = wind->portRect; // the rectangle for this window
										    // in QuickDraw coordinates
	gxShape layoutNormal;				// the regular layout shape we build
	gxShape layoutTight;				// a layout with tighter tracking
	gxShape layoutLoose;				// a layout with looser tracking
	gxShape thePage;					// this window's document shape
	gxRunControls runControls;			// run controls for the layout shape
	gxLayoutOptions layoutOptions;		// options for the layout shape
	
// These are the five text runs for this layout shape, in pieces because they're
// in different languages
	
	char *text = "track track track track track";
	
	gxPoint posn;						// the position of the layout shape


// make default gxLayoutOptions and gxRunControls structures

	InitializeLayoutOptions (&layoutOptions);
	InitializeRunControls (&runControls);
	
	
// Position the layout half way down the left edge of the window. Set 
// the layout's width to the window's width and set the flushness to 1/2, this
// will cause the layout to center in the window.  Note that ourWindowRect is
// not in fixed coordinates, so we have to make it fixed to use it.

	layoutOptions.width = ff(ourWindowRect.right - ourWindowRect.left);
	layoutOptions.flush = fract1/2;

	posn.x = 0;
	posn.y = ff((ourWindowRect.bottom - ourWindowRect.top) / 2);
	

// Create the layouts we'll use.  NewSingleLayout is a library routine found
// in layout library.c to create a layout based on one style.

	layoutNormal = NewSingleLayout(
				text, 							// the text for the layout
				(char *) "\pHoefler Text", 		// the font name to use
				ff(32), 						// the point size to use
				&layoutOptions, 				// our default layout options
				&posn, 							// the shape's position
				0, 								// attributes (none)
				&runControls,					// our run controls
				nil, 							// the run features array (none)
				0, 								// count of zero features = 0
				nil);							// no style run overrides
				
	posn.y -= ff(40);							// move up forty points
	
// Now adjust the tracking to be very tight per the font designer's choices

	runControls.track = ff(-2);
	
	layoutTight = NewSingleLayout(
				text, 							// the text for the layout
				(char *) "\pTimes Roman", 		// the font name to use
				ff(32), 						// the point size to use
				&layoutOptions, 				// our default layout options
				&posn, 							// the shape's position
				0, 								// attributes (none)
				&runControls,					// our run controls
				nil, 							// the run features array (tight)
				0, 								// count of zero features = 0
				nil);							// no style run overrides
				
	posn.y += ff(80);							// now draw forty points below center
	
	
// Now adjust the tracking to be very loose per the font designer's choices

	runControls.track = ff(2);
	
	layoutLoose = NewSingleLayout(
				text, 							// the text for the layout
				(char *) "\pTimes Roman", 		// the font name to use
				ff(32), 						// the point size to use
				&layoutOptions, 				// our default layout options
				&posn, 							// the shape's position
				0, 								// attributes (none)
				&runControls,					// our run controls
				nil, 							// the run features array (loose)
				0, 								// count of zero features = 0
				nil);							// no style run overrides
				
	
// Retrieve the page shape so we can add to it.

	thePage = GetDocShape(wind);
	
// Add the layout to the window's picture shape.  AddToShape is in shape library.c.

	AddToShape(thePage, layoutNormal);
	AddToShape(thePage, layoutTight);
	AddToShape(thePage, layoutLoose);

// Now that all shapes are added to the picture, dispose of them

	GXDisposeShape(layoutNormal);
	GXDisposeShape(layoutTight);
	GXDisposeShape(layoutLoose);
	

// Invalidate the window's portRect so that everything gets updated.
	
	SetPort(wind);
	InvalRect(&ourWindowRect);
}

